/*
 *      CONTENT...Simulated Content of Sample Mean +/- K * Std.Err. Intervals
 */

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <dos.h>
#include <ctype.h>
#include <culproto.h>

#define square(x)  ( (x) * (x) )

#define K_ESC   0x01 /* get1key scan code */
#define QKEY    0x10 /* get1key scan code */
#define NULPTR  (char *) 0              /* null pointer */
#define NUL     '\0'                    /* NUL char */
#define NVAL    0                       /* null value - INTEGER */
#define BEL     0x07                    /* beep */
#define BS      0x08                    /* backspace */
#define ESC     0x1b                    /* Escape */
#define CR      0x0d                    /* carriage return */
#define LF      0x0a                    /* linefeed */
#define RUB     0x7f                    /* delete */
#define NAK     0x15                    /* ^U */
#define ETX     0x03                    /* ^C */
#define CAN     0x18                    /* ^X */
#define Del     0x53                    /* Del key scan code */
#define ECHO    0x8000                  /* echo disable bit */
#define BELL            0x0007          /* ring a ding */
#define LARROW          0x4b00          /* left arrow */
#define RARROW          0x4d00          /* right arrow */
#define DARROW          0x5000          /* down arrow */
#define UARROW          0x4800          /* up arrow */
#define INS             0x5200          /* insert key */
#define DELSC           0x53            /* Del key scan code */
#define DEL             0x5300          /* delete key */
#define HELP            F1              /* same as F1 */
#define BKTAB           0x0f00          /* back (shift) tab */

FILE      *outfile;
char      buf[256], yesno, outnam[13];
int       repl, replics, rstep, ss, sampsize, df;
int       multips, idum, ndx, ccstan;
float     probcont[10000];
double    mean, var, noiseavg, noisevar, noisestd;
double    noise[100], limit[6], acont, avgcont[3], contavg[3];
double    component, fratio, nuo2, xvalue, factor, studcont[3];
WIN_PTR   w0, w1;  /* some windows */
static    double c4[] = { .7979, .8862, .9213, .9400, .9515, .9594,
                          .9650, .9693, .9727, .9754, .9776, .9794,
                          .9810, .9823, .9835, .9845, .9854, .9862,
                          .9869, .9876, .9882, .9887, .9892, .9896 };

void      main( void );
void      handler( void );
long      time( long *time );
int       initseed( void );
double    box_mull( int *idum );
float     whran( int *idum );
char      *win_get( WIN_PTR wn, char *str, int va, char *vcl, int lmax );
void      dblget( double *value );
void      intget( int *value );
void      statval( double value[], int size, double *mean, double *var );

/*
 *  erfc() is the concise "error function," and beta(), betai(), betacf(),
 *  and gammln() are "special functions".  This copyrighted code can be found
 *  in: Press, et.al. (1988). "Numerical Recipes in C",Cambridge Univ.Press.
 */

extern    double erfc( double x );
extern    double beta( double z, double w );
extern    double betai( double a, double b, double x );
extern    double betacf( double a, double b, double x );
extern    double gammln( double xx );

extern    int get1key( void );
extern    int kbhit( void );
extern    void exit( int code );

void handler() {             /* Interrupt (Ctrl-Break) Handler */
        int c;              /* for key "scan code" - character */

        signal(SIGINT,handler);
        win_totop( w1 );
        win_prtf( w1, 1, "\n\t...Hit ESC to quit, any other key to Continue : ");
        c = get1key() >> 8;
        if(c == K_ESC) {
                fcloseall();
                exit(1);
                }
        /* else...normal return, to continue processing... */
        }

/*
 *      initseed()...initialize pseudo-random sequence using the Microsoft
 *                   long time() function to read system clock.
 */

int initseed() {
    long ltime, time();
    int seed;

    time( &ltime );
    seed = (int)ltime ;
    win_prtf( w1, 1, "\n\tUsing Clock to get Seed...\n");
    if( seed > 0 ) return( -seed );
    else if( seed == 0 ) return( -13 );
    else return( seed );
    }

/*
 * Brute Force Box-Muller Transformation.
 */
double box_mull( idum )
int *idum; {

        static int iset=0;
        static double next;
        double rad,ang;

        if( iset == 0 ) {
            rad = sqrt( -2.0 * log( (double)whran( idum ) ) );
            ang = 2.0 * 3.1415926536 * (double)whran( idum );
            next = rad * cos( ang );
            iset = 1;
            return( rad * sin( ang ) );
            }
        else {
            iset=0;
            return next;
            }
        }

/*
 * Random number generator proposed by Wichman and Hill(1982),
 * Algorithm AS 183, Applied Statistics 31, 188-190, Version #2.
 */

float whran( idum )
int *idum; {
        static unsigned long n1, n2, n3;
        static double nn1, nn2, nn3;
        static unsigned long ldum, ix=1, iy=1, iz=1; /* SEEDS */
        static int iff=0;
        double ixx, iyy, izz, rand;

        if( *idum < 0 || iff == 0 ) {
                if( *idum < 0 )
                    *idum *= -1;
                ldum = (unsigned long)(*idum);
                *idum = 1;
                iff=1;
                nn1 = n1 = 30269;
                nn2 = n2 = 30307;
                nn3 = n3 = 30323;
                ix = iy = iz = ldum;
                }
        do {
            ixx = ix = (171*(ix))%n1;
            iyy = iy = (172*(iy))%n2;
            izz = iz = (170*(iz))%n3;
            rand = fmod( ixx/nn1 + iyy/nn2 + izz/nn3, 1.0 );
            } while( rand==0.0 || rand==1.0 );
        return (float)rand;
        }

/*
**  win_get( wn, s, va, cl, l )
**         [window, string, validation atribute, char list, string length]
**
**    va - (1) none               no restrictions
**         (2) integer            0 thru 9 + -
**         (3) floating point     0 thru 9 + - .
**         (4) alpha only         a thru z (upper & lower case)
**         (5) upcase only        A thru Z
**         (6) char list          user supplied validation char list
*/

char *win_get( wn, buf, va, cl, l )     /* get string from window */
WIN_PTR wn;                             /* Window pointer */
char buf[];                             /* user buffer */
int va;                                 /* validation type */
char *cl;                               /* user validation string */
int l; {                                /* maximum number of chars */

  char *p;                              /* local pointer */
  int c;                                /* local character */
  int cc;                               /* character count */

  win_totop(wn);                        /* make sure window is on top */
  p = buf;                              /* init local pointer */
  cc = 0;                               /* init char count */
  *p = NUL;                             /* set up for abort */

  while( cc < l ) {                     /* while string length below max */
    c = get1key();                      /* keyboard Input */
    if( !(c & 0xff) ) {                 /* scan code ?? */
      switch( c >> 8 ) {                /* remapping goes here */
        case DELSC:                     /* Del key */
          c = DEL;                      /* map to delete */
        default:                        /* all others to NOP */
          break;
        }
      }
    c &= 0x7f;                          /* keyboard input */
    if( !c )
      continue;                         /* NUL */
    if( c == ESC || c == CR ) {         /* treat <CR> and ESC the same */
      win_prtf( wn, 1, "%c", CR );
      win_prtf( wn, 1, "%c", LF );      /* print them */
      return( p );                      /* and return */
      }
    if( c == BS || c == DEL ) {         /* check for RUBOUT */
      if( cc <= 0 ) continue;           /* nothing to rub out */
      win_prtf( wn, 1, "\b \b" );       /* delete a char */
      cc--;                             /* decrement char count */
      p--;                              /* decrement pointer */
      *p = NUL;                         /* terminate string */
      continue;                         /* and continue */
      }
    if( c == CAN || c == NAK ||         /* ^X or ^U */
        c == ETX ) {                    /*  or ^C */
      while( cc > 0 ) {                 /* do it till cc is zip */
        win_prtf( wn, 1, "\b \b" );     /* delete a char */
        cc--;                           /* decrement cc */
        p--;                            /* decrement pointer */
        *p = NUL;                       /* terminate string */
        continue;                       /* and continue */
        }
      }
    if( c < ' ' )
      continue;                         /* not printable !! */

    switch( va ) {
      case 1:                           /* no vaidation */
        break;                          /* dont do anything */
      case 2:                           /* integer */
        if( c >= '0' && c <= '9' )      /* valid digit ?? */
          break;                        /* ok.. */
        if( c == '-' || c == '+' )      /* sign ?? */
          break;                        /* ok.. */
        c = NUL;                        /* say ng */
        break;
      case 3:                           /* floating pt */
        if( c >= '0' && c <= '9' )      /* valid digit */
          break;                        /* ok */
        if( c == '-' || c == '+' || c == '.' )
          break;                        /* sign or decimal point */
        c = NUL;                        /* say ng */
        break;
      case 4:                           /* alpha only */
        if( isalpha( c ) )              /* alpha ?? */
          break;                        /* ok */
        c = NUL;                        /* something else */
        break;
      case 5:                           /* upper case only */
        if( isalpha( c ) ) {            /* alpha ?? */
          c = toupper( c );             /* force upper case */
          break;
          }
        c = NUL;                        /* no cigars */
        break;
      case 6:                           /* user supplied list */
        if( strchr( cl, c ) )           /* in the list ?? */
          break;                        /* ok */
        c = NUL;                        /* no cigars */
        break;
      default:                          /* ?? */
        c = NUL;                        /* all others get eaten!! */
        break;
      }
    if( c == NUL ) {                    /* NUL sez validation is ng! */
      win_prtf( wn, 1, "%c", BEL );     /* beep */
      continue;                         /* and continue */
      }
    *p++ = (char)c;                     /* char is ok */
    *p = NUL;                           /* terminate string */
    win_prtf( wn, 0, "%c", c );         /* display the char */
    cc++;                               /* bump count of displayed chars */
    }
  return( p );                          /* return ptr to buffer */
  }

void dblget( value )
double *value; {

        win_get( w1, buf, 3, 0, 9 );
        if( buf[0] != '\0' && buf[0] != '\r' && buf[0] != '\n' )
                sscanf( buf, "%lf", value );
        }

void intget( value )
int *value; {

        win_get( w1, buf, 2, 0, 9 );
        if( buf[0] != '\0' && buf[0] != '\r' && buf[0] != '\n' )
                sscanf( buf, "%d", value );
        }

void statval( value, size, avrg, disp )
double value[];
int size;
double *avrg, *disp; {
   int idx;
   *avrg = value[0];
   *disp = 0.0;
   if( size < 2 )
       return;
   for( idx=1; idx<size; idx++ )
       *avrg += value[idx];
   *avrg /= (double)size;
   for( idx=0; idx<size; idx++ )
       *disp += square( value[idx] - *avrg );
   *disp /= (double)size-1.0;
   }

/* CONTENT main module */

void main() {

   double rroot2;
   extern char wrap_flg; /* declare the text wrapping flag */

   if( signal( SIGINT, handler ) == (int(*)())-1 ) {
           printf( "Couldn't set SIGINT...Abort!\n\n" );
           exit( 1 );
           }

   wrap_flg = TRUE;
   win_init();

   /* set up light source direction */
   curtype(1,0,0);
   set_shadow(1,8,0,0);

   w0 = win_open(0,0,23,78,7,1,1,7,1," Simulated Content ",7,1,"",7,1);
   
   win_prtf( w0, 1, "\n\n\t***  Simulated Content of Sample Mean+/-Std.Err. Intervals  ***\n");
   win_prtf( w0, 1, "\n\n\t                  CONTENT.EXE...Version 9203" );
   win_prtf( w0, 1, "\n\n\t              A Quality Assurance Training Tool:" );
   win_prtf( w0, 1,   "\n\t       Statistics Committee of the QA Section of the PMA" );
   win_prtf( w0, 1, "\n\n\t          Bob Obenchain, CompuServe User [72007,467]\n\n" );
   
   strcpy( outnam, "content" );

   win_prtf( w0, 1, "\n\n\tAt colon Prompts : ...simply press ENTER to get the [default]." );
   
   win_prtf( w0, 1, "\n\n\tSpecify filename for Detailed Output [%s] : ", outnam );
   win_get( w0, buf, 1, 0, 9 );
   if( buf[0] != '\0' && buf[0] != '\r' && buf[0] != '\n' )
           sscanf( buf, "%s", outnam );
   outnam[ min( 8, (int)strcspn( outnam, ". " ) ) ] = '\0';
   if( outnam[0] == '\0' )
           strcpy( outnam, "content" );
   strcat( outnam, ".out" );
   win_prtf( w0, 1, "\n\tThe CONTENT Output Save file is to be: %s\n",
           outnam );
   if( ( outfile = fopen( outnam, "w" ) ) == NULL ) {
           win_prtf( w0, 1, "\tCannot write to Output filename : %s\n", outnam );
           if( ( outfile = fopen( "content.out", "w" ) ) == NULL )
                   win_prtf( w0, 1, "\tCannot write to Output filename : content.out\n" );
           else {
                   win_prtf( w0, 1, "\t...using default Outfile name : content.out\n" );
                   strcpy( outnam, "content.out" );
                   }
           }
   
   win_prtf( w0, 1, "\n\n\t\tPress Q to QUIT now...Other Key to Continue..." );
   if( ( get1key() >> 8 ) == QKEY ) {
           curlocat(24,0);
           printf( "\n" );
           coloreos( 2, 0 );
           exit( 0 );
           }

   w1 = win_open(0,0,23,78,1,7,7,1,7," Parameter Specification ",1,7,"",1,7);
   sampsize = 5;
   ccstan = 2;
   replics = 1000;

   while( 1 ) {

       win_prtf( w1, 1, "\n\n\tSpecify Parameters by Responding to the following prompts...\n" );

       win_prtf( w1, 1, "\n\n\tMaximum Sample Size = 100.");
       win_prtf( w1, 1, "\n\n\tMinimum Sample Size =   2.");
       win_prtf( w1, 1, "\n\n\tEnter Sample Size [%d] : ", sampsize );
       intget( &sampsize );
       if( sampsize < 1 || sampsize > 100 ) {
               win_prtf( w1, 1, "\n\tSample Size will be 5, the default.\n");
               sampsize = 5;
               }

       win_prtf( w1, 1, "\n\n\tSimulate Confidence-in-Content for +/-" );
       win_prtf( w1, 1,   "\n\t1, 2 or 3 Standard Deviation Intervals ?" );
       win_prtf( w1, 1, "\n\n\tEnter Multiplier [%d] : ", ccstan );
       intget( &ccstan );
       if( ccstan < 1 )
           ccstan = 1;
       else if( ccstan > 3 )
           ccstan = 3;

       win_prtf( w1, 1, "\n\n\tMaximum Simulation Replications = 10,000.");
       win_prtf( w1, 1, "\n\n\tEnter Number of Simulation Replications [%d] : ", replics );
       intget( &replics );
       if( replics < 100 || replics > 10000 ) {
               win_prtf( w1, 1, "\n\tNumber of Simulation Replications be 1000, the default.\n");
               replics = 1000;
               }

       win_prtf( w1, 1, "\n\n\tSimulated Content of Sample Mean+/-Std.Err. Intervals..." );
       win_prtf( w1, 1, "\n\tSample Size will be %d.", sampsize );
       win_prtf( w1, 1, "\n\tConfidence-in-Content of +/-%1d Std. Dev. Intervals.", ccstan );
       win_prtf( w1, 1, "\n\tMaximum Number of Simulation Replications will be %d.", replics );

       fprintf( outfile, "\nSimulated Content of Sample Mean+/-Std.Err. Intervals..." );
       fprintf( outfile, "\nSample Size will be %d.", sampsize );
       fprintf( outfile, "\nConfidence-in-Content of +/-%1d Std. Dev. Intervals.", ccstan );
       fprintf( outfile, "\nMaximum Number of Simulation Replications will be %d.", replics );

       idum = 0;
       win_prtf( w1, 1, "\n\n\tInput Start-up Seed ( 0 means use your Clock ): " );
       intget( &idum );
       if( idum > 0 )
               idum = -idum ;
       if( idum == 0 )
               idum = initseed();

       win_prtf( w1, 1, "\n\tRandom Number Seed  = %d", idum );
       fprintf( outfile, "\nRandom Number Seed  = %d", idum );

       for( multips=0; multips<6; multips++ )
               limit[multips] = 0.0;
       avgcont[0] = avgcont[1] = avgcont[2] = 0.0;

       win_totop( w0 );
       win_cls( w0 );

       while( kbhit() )
          get1key();
       win_prtf( w0, 1, "\n\tSimulation Initiated...Press a Key to STOP.\n\n" );

       rroot2 = 1.0 / sqrt( 2.0 );
       df = sampsize - 1;
       nuo2 = (double)df / 2.0;
       factor = (double)sampsize / ( 1.0 + (double)sampsize );

       for( repl = 0; repl < replics; repl++ ) {

           for( ss=0; ss<sampsize; ss++ )
               noise[ss] = box_mull( &idum );
           statval( noise, sampsize, &noiseavg, &noisevar );
           noisestd = sqrt( noisevar );

           limit[0] += noiseavg - noisestd;
           limit[1] += noiseavg + noisestd;
           acont = 0.5 * (   erfc( ( noiseavg - noisestd ) * rroot2 )
                           - erfc( ( noiseavg + noisestd ) * rroot2 ) );
           avgcont[0] += acont;
           if( ccstan == 1 )
               probcont[repl] = (float)acont;

           limit[2] += noiseavg - 2.0 * noisestd;
           limit[3] += noiseavg + 2.0 * noisestd;
           acont = 0.5 * (   erfc( ( noiseavg - 2.0 * noisestd ) * rroot2 )
                           - erfc( ( noiseavg + 2.0 * noisestd ) * rroot2 ) );
           avgcont[1] += acont;
           if( ccstan == 2 )
               probcont[repl] = (float)acont;

           limit[4] += noiseavg - 3.0 * noisestd;
           limit[5] += noiseavg + 3.0 * noisestd;
           acont = 0.5 * (   erfc( ( noiseavg - 3.0 * noisestd ) * rroot2 )
                           - erfc( ( noiseavg + 3.0 * noisestd ) * rroot2 ) );
           avgcont[2] += acont;
           if( ccstan == 3 )
               probcont[repl] = (float)acont;

           if( (repl+1) % 10 == 0 ) {
               win_curset(5,0);
               win_prtf( w0, 1, "\tReplication Number: %d", repl+1 );
               }

           if( kbhit() && repl > 99 ) {
               while( kbhit() )
                  get1key();
               break;
               }
           }

       if( repl < replics )
           replics = repl + 1;
       fprintf( outfile, "\nActual Number of Simulation Replications = %d.", replics );

       for( multips=0; multips<6; multips++ )
               limit[multips] /= (double)replics;

       avgcont[0] /= (double)replics;
       contavg[0] = 0.5 * (   erfc( limit[0] * rroot2 )
                            - erfc( limit[1] * rroot2 ) );
       xvalue = (double)df / ((double)df + factor);
       studcont[0] = 1.0 - betai( nuo2, 0.5, xvalue );

       avgcont[1] /= (double)replics;
       contavg[1] = 0.5 * (   erfc( limit[2] * rroot2 )
                            - erfc( limit[3] * rroot2 ) );
       xvalue = (double)df / ((double)df + 4.0*factor);
       studcont[1] = 1.0 - betai( nuo2, 0.5, xvalue );

       avgcont[2] /= (double)replics;
       contavg[2] = 0.5 * (   erfc( limit[4] * rroot2 )
                            - erfc( limit[5] * rroot2 ) );
       xvalue = (double)df / ((double)df + 9.0*factor);
       studcont[2] = 1.0 - betai( nuo2, 0.5, xvalue );

       win_prtf( w0, 1, "\n\n\tSample Size = %3d", sampsize );
       win_prtf( w0, 1, "\tSimulation Replications = %d.", replics );
       win_prtf( w0, 1, "\n\tC4 estimate = %6.4lf", (limit[1]-limit[0])/2.0 );
       if( sampsize < 26 )
           win_prtf( w0, 1, "\tExact C4    = %6.4lf", c4[sampsize-2] );
       win_prtf( w0, 1, "\n\n\tMean Minus 1 Std. Error  = %7.4lf", limit[0] );
       win_prtf( w0, 1, "  \n\tMean Plus  1 Std. Error  = %7.4lf", limit[1] );
       win_prtf( w0, 1, "  \n\tContent of Avg.+/-1 Interval = %7.4lf", contavg[0] );
       win_prtf( w0, 1, "  \n\tAvg.Content of +/-1 Interval = %7.4lf", avgcont[0] );
       win_prtf( w0, 1, "  \n\tStudent-t Cont.+/-1 Interval = %7.4lf", studcont[0] );
       win_prtf( w0, 1, "\n\n\tMean Minus 2 Std. Errors = %7.4lf", limit[2] );
       win_prtf( w0, 1, "  \n\tMean Plus  2 Std. Errors = %7.4lf", limit[3] );
       win_prtf( w0, 1, "  \n\tContent of Avg.+/-2 Interval = %7.4lf", contavg[1] );
       win_prtf( w0, 1, "  \n\tAvg.Content of +/-2 Interval = %7.4lf", avgcont[1] );
       win_prtf( w0, 1, "  \n\tStudent-t Cont.+/-2 Interval = %7.4lf", studcont[1] );
       win_prtf( w0, 1, "\n\n\tMean Minus 3 Std. Errors = %7.4lf", limit[4] );
       win_prtf( w0, 1, "  \n\tMean Plus  3 Std. Errors = %7.4lf", limit[5] );
       win_prtf( w0, 1, "  \n\tContent of Avg.+/-3 Interval = %7.4lf", contavg[2] );
       win_prtf( w0, 1, "  \n\tAvg.Content of +/-3 Interval = %7.4lf", avgcont[2] );
       win_prtf( w0, 1, "  \n\tStudent-t Cont.+/-3 Interval = %7.4lf", studcont[2] );
       win_prtf( w0, 1, "\n\n\tSorting Simulation Results..." );

       fprintf( outfile, "\n\nC4 estimate = %6.4lf", (limit[1]-limit[0])/2.0 );
       if( sampsize < 26 )
           fprintf( outfile, "\nExact C4    = %6.4lf", c4[sampsize-2] );
       fprintf( outfile, "  \nMean Minus 1 Std. Error  = %7.4lf", limit[0] );
       fprintf( outfile, "  \nMean Plus  1 Std. Error  = %7.4lf", limit[1] );
       fprintf( outfile, "  \nContent of Avg.+/-1 Interval = %7.4lf", contavg[0] );
       fprintf( outfile, "  \nAvg.Content of +/-1 Interval = %7.4lf", avgcont[0] );
       fprintf( outfile, "  \nStudent-t Cont.+/-1 Interval = %7.4lf", studcont[0] );
       fprintf( outfile, "\n\nMean Minus 2 Std. Errors = %7.4lf", limit[2] );
       fprintf( outfile, "  \nMean Plus  2 Std. Errors = %7.4lf", limit[3] );
       fprintf( outfile, "  \nContent of Avg.+/-2 Interval = %7.4lf", contavg[1] );
       fprintf( outfile, "  \nAvg.Content of +/-2 Interval = %7.4lf", avgcont[1] );
       fprintf( outfile, "  \nStudent-t Cont.+/-2 Interval = %7.4lf", studcont[1] );
       fprintf( outfile, "\n\nMean Minus 3 Std. Errors = %7.4lf", limit[4] );
       fprintf( outfile, "  \nMean Plus  3 Std. Errors = %7.4lf", limit[5] );
       fprintf( outfile, "  \nContent of Avg.+/-3 Interval = %7.4lf", contavg[2] );
       fprintf( outfile, "  \nAvg.Content of +/-3 Interval = %7.4lf", avgcont[2] );
       fprintf( outfile, "  \nStudent-t Cont.+/-3 Interval = %7.4lf", studcont[2] );

       fsort( probcont, replics, compfa );

       win_prtf( w0, 1, "\n\tPress a Key to Continue..." );
       get1key();

       win_prtf( w0, 1, "\n\n\tConfidence-in-Content of +/-%1d Std. Dev. Intervals...", ccstan );
       win_prtf( w0, 1, "\n\tSimulation Replications = %d.", replics );
       win_prtf( w0, 1, "\n\tResults:\n" );
       fprintf( outfile, "\n\nConfidence-in-Content Results..." );
       rstep = replics/20;
       ss = 105;
       for( repl = 0; repl <= replics/2; repl += rstep ) {
          ss -= 5;
          win_prtf( w0, 1, "\n\tContent Order Statistic No. %4d = %7.4f  Confidence =%3d%%",
              repl+1, probcont[repl], ss );
          fprintf( outfile, "\nContent Order Statistic No. %4d = %7.4f  Confidence =%3d%%",
              repl+1, probcont[repl], ss );
          }
       fprintf( outfile, "\n==========================================================================\n\n" );

       win_prtf( w0, 1, "\n\n\t\tPress the Q key to QUIT now..." );
       win_prtf( w0, 1, "\n\t\tPress some Other Key to Start Another Simulation..." );
       if( ( get1key() >> 8 ) == QKEY )
           break;

       win_totop( w1 );
       win_cls( w1 );
       }

   fcloseall();
   curlocat(24,0);
   printf( "\n" );
   coloreos( 2, 0 );
   
   if( outfile != NULL ) {
           printf( "\n\nREMINDER:\n" );
           printf( "\nCONTENT created an Output File named... %s\n", outnam );
           printf( "\n\tUse the DOS invocation... TYPE %s | MORE", outnam );
           printf( "\n\tto review detailed computational results from" );
           printf( "\n\tthis CONTENT run.\n" );
       }
   
   printf( "\n" );
   exit( 0 );
   }
